using System;
using System.IO;
using System.Diagnostics;

using MediaPortal.Player;
using MediaPortal.GUI.Library;
using MediaPortal.Util;

namespace MediaPortal.Plugins
{

  public class RarPlayerPlugin : IExternalPlayer, ISetupForm
  {

    #region Variables

    string _currentPlayedFile = null;

    #endregion Variables

    #region IExternalPlayer Members

    public override string AuthorName                     { get { return "and-81 and jackjson"; } }
    public override string[] GetAllSupportedExtensions()  { return new string[] { ".rar" }; }
    public override string PlayerName                     { get { return "RarPlayerMount"; } }
    public override bool SupportsFile(string filename)    { return filename.EndsWith(".rar", StringComparison.InvariantCultureIgnoreCase); }
    public override string VersionNumber                  { get { return "2.4.0.0"; } }

    #endregion IExternalPlayer Members

    #region ISetupForm Members

    public new bool CanEnable()           { return true; }
    public new bool DefaultEnabled()      { return true; }
    public override string Description()  { return "Mount RAR files for playback"; }
    public new bool HasSetup()            { return false; }
    public override void ShowPlugin()     { }

    #endregion

    #region IPlayer Members

    public override bool Play(string strFile)
    {

      string movieFilename = MountRar(strFile);
      string strVirtualDrive = "";

      if (movieFilename.Length > 0 && File.Exists(movieFilename))
      {
        _currentPlayedFile = movieFilename;
        GUIWindowManager.Receivers += new SendMessageHandler(GUIWindowManager_Receivers);

        if (DaemonTools.IsEnabled)
        {
            string extension = Path.GetExtension(movieFilename);
            if (DaemonTools.IsImageFile(extension))
            {
                //MediaPortal.Ripper.AutoPlay.StartListening();
                DaemonTools.Mount(movieFilename, out strVirtualDrive);
            }
            else
            {
                PlayFile(movieFilename);
            }
        }
        else
        {
            PlayFile(movieFilename);
        }
      }

      return false;
    }

    #endregion IPlayer Members

    #region Implementation

    void GUIWindowManager_Receivers(GUIMessage message)
    {
      if (message.Message == GUIMessage.MessageType.GUI_MSG_PLAYBACK_STOPPED || message.Message == GUIMessage.MessageType.GUI_MSG_PLAYBACK_ENDED)
      {
        if (!String.IsNullOrEmpty(_currentPlayedFile))
        {
          Log.Info("RarPlayer: Unmounting file {0}", _currentPlayedFile);

          if (DaemonTools.IsEnabled)
          {
              if (DaemonTools.IsMounted(_currentPlayedFile))
              {
                  DaemonTools.UnMount();
                  // Wait 2 sec
                  for (int i = 0; i < 20; i++)
                      System.Threading.Thread.Sleep(100);
              }
          }

          UnmountRar();
          
          _currentPlayedFile = null;
        }

        GUIWindowManager.Receivers -= new SendMessageHandler(GUIWindowManager_Receivers);
      }
    }

    string MountRar(string filename)
    {
      Log.Info("RarPlayer: Mount file {0}", filename);


      //MediaPortal.Util.DaemonTools.IsImageFile
      // Check for mounted drive
      try
      {
          if (System.IO.Directory.Exists(@"M:\"))
          {
              Log.Info("RarPlayer: RAR file already mounted. Unmounting.");
              if (DaemonTools.IsEnabled)
              {
                  DaemonTools.UnMount();
                  // Wait 2 sec
                  for (int i = 0; i < 20; i++)
                      System.Threading.Thread.Sleep(100);
              }
              UnmountRar();
              //return "";
              // Wait 3 sec
              for (int i = 0; i < 30; i++)
                  System.Threading.Thread.Sleep(100);
          }
      }
      catch
      {
          Log.Info("RarPlayer: Expected error, file doesn't exist yet: {0}", filename);
      }

      string rararguments = String.Format("/silent \"{0}\"", filename);
      Process rarprocess = new Process();
      rarprocess.StartInfo.FileName = "RarMount2.exe";
      rarprocess.StartInfo.WorkingDirectory = ".";
      rarprocess.StartInfo.Arguments = rararguments;
      rarprocess.StartInfo.WindowStyle = ProcessWindowStyle.Minimized;
      //rarprocess.StartInfo.CreateNoWindow = true;
      rarprocess.StartInfo.UseShellExecute = false;
      rarprocess.Start();

      // Wait 3 sec
      for (int i = 0; i < 30; i++)
          System.Threading.Thread.Sleep(100);

      // imdisk -a -t proxy -m #: -o ro,rem,ip -f 127.0.0.1:10230 -b 0 -s 5G
      string mountarguments = "-a -t proxy -m M: -u 5 -o ro,rem,ip -f 127.0.0.1:32123 -b 0 -s 31G";
      Process mountprocess = new Process();
      mountprocess.StartInfo.FileName = "imdisk.exe";
      mountprocess.StartInfo.WorkingDirectory = ".";
      mountprocess.StartInfo.Arguments = mountarguments;
      mountprocess.StartInfo.WindowStyle = ProcessWindowStyle.Minimized;
      //mountprocess.StartInfo.CreateNoWindow = true;
      mountprocess.StartInfo.UseShellExecute = false;
      mountprocess.Start();
      mountprocess.WaitForExit();

      DirectoryInfo dirInfo = new DirectoryInfo("M:\\");
      FileInfo[] dirFiles = dirInfo.GetFiles("UNPACKED.*");
      if (dirFiles.Length > 0)
          return dirFiles[0].FullName;
      else
          return "";
    }

    void UnmountRar()
    {
      string mountarguments = "-d -u 5";
      Process mountprocess = new Process();
      mountprocess.StartInfo.FileName = "imdisk.exe";
      mountprocess.StartInfo.WorkingDirectory = ".";
      mountprocess.StartInfo.Arguments = mountarguments;
      mountprocess.StartInfo.WindowStyle = ProcessWindowStyle.Minimized;
      //mountprocess.StartInfo.CreateNoWindow = true;
      mountprocess.StartInfo.UseShellExecute = false;
      mountprocess.Start();
      mountprocess.WaitForExit();
    }

    void PlayFile(string movieFilename)
    {
      Log.Info("RarPlayer: Play file {0}", movieFilename);

      GUIMessage message = new GUIMessage(GUIMessage.MessageType.GUI_MSG_PLAY_FILE, 0, 0, 0, 0, 0, null);
      message.Label = movieFilename;

      GUIGraphicsContext.ResetLastActivity();
      GUIWindowManager.SendThreadMessage(message);
      GUIWindowManager.SendThreadMessage(new GUIMessage(GUIMessage.MessageType.GUI_MSG_GOTO_WINDOW, 0, 0, 0, (int)GUIWindow.Window.WINDOW_FULLSCREEN_VIDEO, 0, null));
    }

    #endregion Implementation

  }

}
